#ifndef _Plane_H
#define _Plane_H 1

/** @class Plane 
 *
 * Define a detector plane with longitudinal 
 * strips of pitch defined in 'constants.h'
 *
 * Plane is defined in (u,v) frame with u the
 * strip direction.
 *
 *
 *  @author M.Needham
 */

#include "constants.h"
#include <iostream>
#include <math.h>

class Plane {

public:

  Plane(const double uL, const double uR,
        const double vD, const double vU,
        const double z):
  m_uL(uL),
  m_uR(uR),
  m_vD(vD),
  m_vU(vU), 
  m_z(z){

    m_NumStrips = (int)((m_uL - m_uR)/pitch);
  
  }

  ~Plane(){}

  double uL() const {return m_uL;}

  double uR() const {return m_uR;}

  double vD() const {return m_vD;}
 
  double vU() const {return m_vU;}

  double z() const {return m_z;}

  bool isInside(const double u, const double v) const ;

  unsigned int strip(const double umeas) const;

  bool isStrip(const unsigned int Strip) const;

  double U(const unsigned int Strip) const ;

 private:

  double m_uL;  // Left plane end (in u) 
  double m_uR;  // Right plane end (in u) 
  double m_vD;  // Left plane end (in v) 
  double m_vU;  // Right plane end (in v) 
  double m_z;   // z position of the plane (defined using constants.h)
  unsigned int m_NumStrips;


};


#include <vector>
typedef std::vector<Plane> Planes;

inline bool Plane::isInside(const double u, const double v) const{

 // check inside module
 
 return (((v > m_vD)&&(v < m_vU)) && ((u > m_uL)&&(u<m_uR))) ;
}

inline unsigned int Plane::strip(const double umeas) const{
  return (unsigned int)floor((umeas-m_uL)/pitch);
}

inline double Plane::U(const unsigned int strip) const {

  double u = -999.;
  if (isStrip(strip)) {
    u = m_uL + (double(strip)*pitch);
  }

  return u;
}

inline bool Plane::isStrip(const unsigned int Strip) const {
  return (Strip >= 0 && Strip < m_NumStrips);
}

#endif
